#include "js-gtk.h"

/* Horizontal box */

JSGTK_CLASS(HBox);

static JSBool hbox_ctor(JSContext *cx, JSObject *obj,
			uintN argc, jsval *argv, jsval *rval) {
    GtkWidget *hbox;
    uint32 arg1;
    
    if (argc < 2 ||
	!JSVAL_IS_BOOLEAN(argv[0]) ||
	!JSVAL_IS_INT(argv[1]) ||
	!JS_IsConstructing(cx)) {
	JS_ReportError(cx, "Bad types for GtkHBox: %s,%s",
		       JS_GetTypeName(cx, JS_TypeOfValue(cx, argv[0])),
		       JS_GetTypeName(cx, JS_TypeOfValue(cx, argv[1])));
	return JS_FALSE;
    }
    
    if (!JS_ValueToInt32(cx, argv[0], &arg1))
	return JS_FALSE;
    hbox = gtk_hbox_new(arg1, JSVAL_TO_INT(argv[1]));
    JS_SetPrivate(cx, obj, hbox);
    jsgtkobj_registerWrapper(cx, GTK_OBJECT(hbox), obj);
    return JS_TRUE;
}

#define hbox_methods NULL
JSGTK_INIT_FUNC(hbox,HBox,2);

/* Vertical box */

JSGTK_CLASS(VBox);

static JSBool vbox_ctor(JSContext *cx, JSObject *obj,
			uintN argc, jsval *argv, jsval *rval) {
    GtkWidget *vbox;
    uint32 arg1;
    
    if (argc < 2 ||
	!JSVAL_IS_BOOLEAN(argv[0]) ||
	!JSVAL_IS_INT(argv[1]) ||
	!JS_IsConstructing(cx)) {
	JS_ReportError(cx, "Bad types for GtkVBox: %s,%s",
		       JS_GetTypeName(cx, JS_TypeOfValue(cx, argv[0])),
		       JS_GetTypeName(cx, JS_TypeOfValue(cx, argv[1])));
	return JS_FALSE;
    }
    
    if (!JS_ValueToInt32(cx, argv[0], &arg1))
	return JS_FALSE;
    vbox = gtk_vbox_new(arg1, JSVAL_TO_INT(argv[1]));
    JS_SetPrivate(cx, obj, vbox);
    jsgtkobj_registerWrapper(cx, GTK_OBJECT(vbox), obj);
    return JS_TRUE;
}

#define vbox_methods NULL
JSGTK_INIT_FUNC(vbox,VBox,2);

/* Base box class */

#define WIDGET_FUNC(funcname) \
  WIDGET_WRAPPER_FUNC(box,Box,BOX,funcname)

WIDGET_FUNC(pack_start_defaults)
WIDGET_FUNC(pack_end_defaults);

#undef WIDGET_FUNC

#define PACK_FUNC(name)\
static JSBool box_pack_##name(JSContext *cx, JSObject *obj,                   \
			    uintN argc, jsval *argv, jsval *rval) {           \
    GtkBox *box;                                                              \
    GtkWidget *widget;                                                        \
    int32 arg1, arg2;	                                                      \
    if (argc < 4 ||                                                           \
	!JSVAL_IS_OBJECT(argv[0]) ||                                          \
	!JSVAL_IS_BOOLEAN(argv[1]) || !JSVAL_IS_BOOLEAN(argv[2]) ||           \
	!JSVAL_IS_INT(argv[3])) {                                             \
        JS_ReportError(cx, "bad types to GtkBox." #name ": %d/%s\n",          \
		       argc, jsgtk_print_types(cx, argv, argc));              \
	return JS_FALSE;                                                      \
    }                                                                         \
    GET_GTK(box,cx,obj,BOX);                                                  \
    GET_GTK(widget,cx,JSVAL_TO_OBJECT(argv[0]),WIDGET);                       \
    if (!JS_ValueToInt32(cx, argv[1], &arg1) ||                               \
	!JS_ValueToInt32(cx, argv[2], &arg2))                                 \
	return JS_FALSE;                                                      \
    gtk_box_pack_##name(box, widget, arg1, arg2, JSVAL_TO_INT(argv[3]));      \
    return JS_TRUE;                                                           \
}

PACK_FUNC(start)
PACK_FUNC(end);

#undef PACK_FUNC

JSGTK_CLASS(Box);

static JSFunctionSpec box_methods[] = {
    {"pack_start_defaults",	box_pack_start_defaults,	1},
    {"pack_end_defaults",	box_pack_end_defaults,		1},
    {"pack_end",		box_pack_end,			4},
    {"pack_start",		box_pack_start,			4},
    {0}
};

JSBool jsgtkhbox_init(JSContext *cx, JSObject *global, JSObject *gtkproto);
JSBool jsgtkvbox_init(JSContext *cx, JSObject *global, JSObject *gtkproto);

JSBool jsgtkbox_init(JSContext *cx, JSObject *global, JSObject *gtkproto) {
    JSObject *boxProto;
    
    boxProto = JS_InitClass(cx, global, gtkproto, &jsgtk_BoxClass,
			    NULL, 0,
			    NULL, box_methods,
			    NULL, NULL);
    if (!boxProto ||
	!jsgtkhbox_init(cx, global, boxProto) ||
	!jsgtkvbox_init(cx, global, boxProto))
	return JS_FALSE;
    
    return JS_TRUE;
}
