#include "js-gtk.h"

JSGTK_CLASS(Entry);

static JSBool entry_set_text(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkEntry *entry;
  
  if (argc < 1 || !JSVAL_IS_STRING(argv[0]))
      return JS_FALSE;
  
  GET_GTK(entry,cx,obj,ENTRY);
  gtk_entry_set_text(entry, JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
  return JS_TRUE;

}
  
static JSBool entry_get_text(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkEntry *entry;
  gchar *str;
  
  GET_GTK(entry,cx,obj,ENTRY);
  str = gtk_entry_get_text(entry);
  *rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, str));
  return JS_TRUE;
}

static JSBool entry_append_text(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkEntry *entry;
  
  if (argc < 1 || !JSVAL_IS_STRING(argv[0]))
      return JS_FALSE;
  
  GET_GTK(entry,cx,obj,ENTRY);
  gtk_entry_append_text(entry, JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
  return JS_TRUE;
}

static JSBool entry_prepend_text(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkEntry *entry;
  
  if (argc < 1 || !JSVAL_IS_STRING(argv[0]))
      return JS_FALSE;
  
  GET_GTK(entry,cx,obj,ENTRY);
  gtk_entry_prepend_text(entry, JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
  return JS_TRUE;
}

static JSBool entry_set_position(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkEntry *entry;
  
  if (argc < 1 || !JSVAL_IS_INT(argv[0]))
      return JS_FALSE;
  
  GET_GTK(entry,cx,obj,ENTRY);
  gtk_entry_set_position(entry, JSVAL_TO_INT(argv[0]));
  return JS_TRUE;
}

static JSBool entry_select_region(JSContext *cx, JSObject *obj,
			     uintN argc, jsval *argv, jsval *rval) {  
  GtkEntry *entry;
  
  if (argc < 2 || !JSVAL_IS_INT(argv[0]) ||
      !JSVAL_IS_INT(argv[1]))
      return JS_FALSE;
  
  GET_GTK(entry,cx,obj,ENTRY);
  gtk_entry_select_region(entry, JSVAL_TO_INT(argv[0]), JSVAL_TO_INT(argv[1]));
  return JS_TRUE;
}


static JSBool entry_ctor (JSContext *cx, JSObject *obj,
			 uintN argc, jsval *argv, jsval *rval) {
    GtkWidget *entry;
     
    
    entry = gtk_entry_new();
    JS_SetPrivate(cx,obj, entry);
    jsgtkobj_registerWrapper(cx, GTK_OBJECT(entry), obj);
    return JS_TRUE;
}

static JSFunctionSpec entry_methods[] = {
    {"set_text", entry_set_text, 1},
    {"get_text", entry_get_text, 0},
    {"append_text", entry_append_text, 1},
    {"prepend_text", entry_prepend_text, 1},
    {"set_position", entry_set_position, 1},
    {"select_region", entry_select_region, 2},
    {0}
};

JSGTK_INIT_FUNC(entry,Entry,0);

