#ifndef JS_GTK_H
#define JS_GTK_H

#include <jsapi.h>
#include <gtk/gtk.h>

JSBool jsgtkobj_registerWrapper(JSContext *, GtkObject *, JSObject *);
JSObject *jsgtkobj_findWrapper(JSContext *, GtkObject *);
JSBool jsgtkobj_unregisterWrapper(JSContext *, GtkObject *);

void jsgtkobj_finalize(JSContext *, JSObject *);

JSBool jsgtkobj_init(JSContext *, JSObject *, int, char **);
char *jsgtk_print_types(JSContext *cx, jsval *argv, uintN argc);

#define GET_GTK(var,cx,obj,TYPE) \
var = JS_GetPrivate(cx, obj);    \
if (!GTK_IS_##TYPE(var)) {       \
  JS_ReportError(cx, "%s " #var " isn't a " #TYPE ": %s", \
		 JS_GetClass(obj)->name, \
		 gtk_type_name(GTK_OBJECT(var)->klass->type)); \
  return JS_FALSE; \
}

/* SIMPLE_WRAPPER_FUNC(container,Container,CONTAINER,funcname)*/
#define SIMPLE_WRAPPER_FUNC(prefix,Type,TYPE,funcname)                        \
static JSBool prefix##_##funcname(JSContext *cx, JSObject *obj,               \
                                  uintN argc, jsval *argv, jsval *rval) {     \
  Gtk##Type *prefix;                                                          \
  GET_GTK(prefix,cx,obj,TYPE);                                                \
  gtk_##prefix##_##funcname(prefix);                                          \
  return JS_TRUE;                                                             \
}

#define INT_WRAPPER_FUNC(prefix,Type,TYPE,funcname)                           \
static JSBool prefix##_##funcname(JSContext *cx, JSObject *obj,               \
                                  uintN argc, jsval *argv, jsval *rval) {     \
    Gtk##Type *prefix;                                                        \
    if (argc < 1 || !JSVAL_IS_INT(argv[0]))                                   \
	return JS_FALSE;                                                      \
    GET_GTK(prefix,cx,obj,TYPE);                                              \
    gtk_##prefix##_##funcname(prefix, JSVAL_TO_INT(argv[0]));                 \
    return JS_TRUE;                                                           \
}

#define WIDGET_WRAPPER_FUNC(prefix,Type,TYPE,funcname)                        \
static JSBool prefix##_##funcname(JSContext *cx, JSObject *obj,               \
			          uintN argc, jsval *argv, jsval *rval) {     \
    Gtk##Type *prefix;                                                        \
    GtkWidget *_widget;                                                       \
    if (argc < 1 || !JSVAL_IS_OBJECT(argv[0]))                                \
	return JS_FALSE;                                                      \
    GET_GTK(prefix,cx,obj,TYPE);                                              \
    GET_GTK(_widget,cx,JSVAL_TO_OBJECT(argv[0]),WIDGET);                      \
    gtk_##prefix##_##funcname(prefix, _widget);                               \
    return JS_TRUE;                                                           \
}

#define JSGTK_CLASS(type) \
JSClass jsgtk_##type##Class = {                                               \
    "Gtk" #type,                                                              \
    JSCLASS_HAS_PRIVATE,                                                      \
    JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,  JS_PropertyStub,    \
    JS_EnumerateStub, JS_ResolveStub,   JS_ConvertStub,   jsgtkobj_finalize   \
};

/* JG 1999 02 27 */
/* In the macro below, I had to put JSOJect * type##Proto because
the compiler complained about xxxinit not defined
*/

#define JSGTK_INIT_FUNC(type,Type,nargs) \
JSObject *jsgtk_##Type##Proto;                                                  \
JSBool jsgtk##type##_init(JSContext *cx, JSObject *global,                    \
			  JSObject *gtkproto) { \
    JSObject  * type##Proto;  \
    type##Proto = JS_InitClass(cx, global, gtkproto, &jsgtk_##Type##Class,    \
			       type##_ctor, nargs,                            \
			       NULL, type##_methods,                          \
			       NULL, NULL);                                   \
    if (! type##Proto)                                                         \
	return JS_FALSE;                                                      \
    return JS_TRUE;                                                           \
}

#define JSGTK_CTOR(type) \
static JSBool  type##_ctor(JSContext *cx, JSObject *obj,               \
                                  uintN argc, jsval *argv, jsval *rval) {     \
  GtkWidget * widget;                                                          \
  widget = gtk_##type##_new();                        \
  JS_SetPrivate(cx,obj,widget);                               \
  jsgtkobj_registerWrapper(cx, GTK_OBJECT(widget), obj);         \
  return JS_TRUE;                                                             \
} 

#define JSGTK_CTOR(type) \
static JSBool  type##_ctor(JSContext *cx, JSObject *obj,               \
                                  uintN argc, jsval *argv, jsval *rval) {     \
  GtkWidget * widget;                                                          \
  widget = gtk_##type##_new();                        \
  JS_SetPrivate(cx,obj,widget);                               \
  jsgtkobj_registerWrapper(cx, GTK_OBJECT(widget), obj);         \
  return JS_TRUE;                                                             \
} 


#define JSGTK_FUNCS_BEG(type) \
static JSFunctionSpec type##_methods[] = { 

#define JSGTK_FUNCS_END() \
{0} \
};


#endif /* !JS_GTK_H */
